@@ -147,7 +147,6 @@ struct _MetaWindow
   
   /* whether net_wm_user_time has been set yet */
   guint net_wm_user_time_set : 1;
-  guint focus_despite_user_time : 1;
   
   /* These are the flags from WM_PROTOCOLS */
   guint take_focus : 1;
@@ -386,6 +385,9 @@ void        meta_window_resize_with_gravity (MetaWindow  *window,
 void        meta_window_fill_horizontal     (MetaWindow  *window);
 void        meta_window_fill_vertical       (MetaWindow  *window);
 
+/* Return whether the window should be currently mapped */
+gboolean    meta_window_should_be_showing   (MetaWindow  *window);
+
 /* This recalcs the window/frame size, and recalcs the frame
  * size/contents as well.
  */
@@ -418,6 +420,9 @@ void        meta_window_get_geometry         (MetaWindow  *window,
                                               int         *height);
 void        meta_window_get_outer_rect       (MetaWindow    *window,
                                               MetaRectangle *rect);
+void        meta_window_get_xor_rect         (MetaWindow          *window,
+                                              const MetaRectangle *grab_wireframe_rect,
+                                              MetaRectangle       *xor_rect);
 void        meta_window_delete             (MetaWindow  *window,
                                             Time         timestamp);
 void        meta_window_kill               (MetaWindow  *window);
